@@ -21,12 +21,13 @@ class AsyncWebsocketStreamInterface(metaclass=ABCMeta):
         self._wsq.present_ws: websockets.WebSocketClientProtocol = None
         self._wsq.previous_ws: websockets.WebSocketClientProtocol = None
         self._exiting = False
-        asyncio.create_task(self._ws_manager())
+        self._ws_manager_task = asyncio.create_task(self._ws_manager())
         self._handlers = set()
         # 更换ws连接时期和处理ws数据时期，刚开始是exchanging_ws时期
         AsyncExclusivePeriod.create_obj_periods(self, 'exchanging_ws', 'handing_ws')
         self._when2create_new_ws_task = None
         self._handle_raw_ws_msg_task = None
+        self._handle_wsq_task: asyncio.Task = None
 
     async def _handle_wsq(self):
         '''
@@ -101,29 +102,36 @@ class AsyncWebsocketStreamInterface(metaclass=ABCMeta):
 
     async def exit(self):
         self._exiting = True
+        self._ws_manager_task.cancel()
+        self._handle_wsq_task.cancel()
         if self._wsq.present_ws:
             await asyncio.create_task(self._wsq.present_ws.close())
 
     async def _ws_manager(self):
         # 启动ws连接队列的消息对接handlers处理任务
-        asyncio.create_task(self._handle_wsq())
+        self._handle_wsq_task = asyncio.create_task(self._handle_wsq())
         initail = True
         while not self._exiting or initail:
-            initail = False
-            new_ws = await self._create_ws()
-            # 更新连接
-            self._wsq.put_nowait(new_ws)
-            # wait until ws is exchanged.
-            await AsyncExclusivePeriod.wait_enter_period(self, 'handing_ws')
-            logger.debug('New ws connection opened.')
+            try:
+                initail = False
+                new_ws = await self._create_ws()
+                # 更新连接
+                self._wsq.put_nowait(new_ws)
+                # wait until ws is exchanged.
+                await AsyncExclusivePeriod.wait_enter_period(self, 'handing_ws')
+                logger.debug('New ws connection opened.')
 
-            # 等待需要更新连接的信号或者连接报错
-            self._when2create_new_ws_task = asyncio.create_task(self._when2create_new_ws())
-            await asyncio.wait([self._when2create_new_ws_task])
-            # 进入更换ws时期
-            AsyncExclusivePeriod.set_obj_period(self, 'exchanging_ws')
-            if self._handle_raw_ws_msg_task:
-                self._handle_raw_ws_msg_task.cancel()
+                # 等待需要更新连接的信号或者连接报错
+                self._when2create_new_ws_task = asyncio.create_task(self._when2create_new_ws())
+                await asyncio.wait([self._when2create_new_ws_task])
+                # 进入更换ws时期
+                AsyncExclusivePeriod.set_obj_period(self, 'exchanging_ws')
+                if self._handle_raw_ws_msg_task:
+                    self._handle_raw_ws_msg_task.cancel()
+            except asyncio.CancelledError:
+                raise
+            except:
+                await asyncio.create_task(asyncio.sleep(0))
 
     def stream_filter(self, _filters: list = None):
         '''
@@ -193,7 +201,8 @@ class AsyncWebsocketStreamInterface(metaclass=ABCMeta):
     @abstractmethod
     async def _when2create_new_ws(self):
         '''
-        One time check to notice that it is time to exchange the ws.
+        When the coroutine is done,no matter raised error or not, it is time to exchange the ws.
+
         :return:
         '''
         pass
